@@ -3,8 +3,9 @@
 @author:XuMing(xuming624@qq.com)
 @description: 
 """
-__version__ = '1.0.7'
+__version__ = '1.2.0'
 
-from parrots import utils
 from parrots.asr import SpeechRecognition
 from parrots.pinyin2hanzi import Pinyin2Hanzi
+from parrots.tts import TextToSpeech
+from parrots.log import logger, set_log_level, add_file_logger, LOG_LEVEL
\ No newline at end of file
